﻿//-----------------------------------------------------------------------
// <author email="wen.haixiong@live.com" blog="http://www.cnblogs.com/cesium/">温海雄</author>
//-----------------------------------------------------------------------
namespace OpenSesame
{
    using System;
    using System.Collections.Generic;

    [Serializable]
    public class BookmarkManager
    {
        Dictionary<string, Bookmark> m_Bookmarks = new Dictionary<string, Bookmark>();

        public void Add(Bookmark bookmark)
        {
            if (bookmark == null)
                throw new ArgumentNullException("bookmark");

            m_Bookmarks.Add(bookmark.Name, bookmark);
        }

        public void Remove(Bookmark bookmark)
        {
            if (bookmark == null)
                throw new ArgumentNullException("bookmark");

            m_Bookmarks.Remove(bookmark.Name);
        }

        public void Resume(string bookmarkName, object payload)
        {
            if (bookmarkName == null)
                throw new ArgumentNullException("bookmarkName");

            if (m_Bookmarks.ContainsKey(bookmarkName))
            {
                var bookmark = m_Bookmarks[bookmarkName];
                bookmark.Payload = payload;
                bookmark.ContinueAt(bookmark);
                m_Bookmarks.Remove(bookmarkName);
                return;
            }

            throw new BookmarkNotFoundException();
        }

        public void RunProgramStatement(ProgramStatement statement, BookmarkLocation continueAt)
        {
            Bookmark bookmark = new Bookmark(statement.ToString(), continueAt);
            statement.Run(this);
            bookmark.BookmarkManager = this;
            Add(bookmark);
        }
    }
}
