﻿//-----------------------------------------------------------------------
// <author email="wen.haixiong@live.com" blog="http://www.cnblogs.com/cesium/">温海雄</author>
//-----------------------------------------------------------------------
namespace OpenSesame
{
    using System;
    using System.Collections.Generic;
    using System.Data;
    using System.Data.OleDb;

    internal static class DbHelper
    {
        private static OleDbConnection _DefaultDbConnection;

        internal static OleDbConnection DefaultDbConnection
        {
            get
            {
                if (_DefaultDbConnection == null)
                {
                    lock (typeof(DbHelper))
                    {
                        if (_DefaultDbConnection == null)
                        {
                            _DefaultDbConnection = new OleDbConnection(@"Provider=Microsoft.Jet.OLEDB.4.0;Data Source=..\..\Database\MythicalRuntime.mdb");
                        }
                    }
                }

                return _DefaultDbConnection;
            }
        }

        internal static void InsertProgramHandle(ProgramHandle handle)
        {
            if (handle == null)
                throw new ArgumentNullException("handle");

            OleDbCommand command = new OleDbCommand("INSERT INTO ProgramHandle VALUES(@id,@instance,@created)", DefaultDbConnection);
            command.Parameters.Add(new OleDbParameter("id", OleDbType.VarChar) { Value = handle.ProgramId.ToString() });
            command.Parameters.Add(new OleDbParameter("instance", OleDbType.LongVarChar) { Value = handle.ProgramId.ToString() });
            command.Parameters.Add(new OleDbParameter("created", OleDbType.Date) { Value = DateTime.Now });
            DefaultDbConnection.Open();
            try
            {
                command.ExecuteNonQuery();
            }
            finally
            {
                DefaultDbConnection.Close();
            }
        }

        internal static bool ExistsProgramHandle(ProgramHandle handle)
        {
            if (handle == null)
                throw new ArgumentNullException("handle");

            OleDbCommand command = new OleDbCommand("SELECT id FROM ProgramHandle WHERE id=@id", DefaultDbConnection);
            command.Parameters.Add(new OleDbParameter("id", OleDbType.VarChar) { Value = handle.ProgramId.ToString() });
            DefaultDbConnection.Open();
            try
            {
                using (OleDbDataReader dataResder = command.ExecuteReader())
                {
                    if (dataResder.Read())
                        return true;

                    return false;
                }
            }
            finally
            {
                DefaultDbConnection.Close();
            }
        }

        internal static void DeleteProgramHandle(ProgramHandle handle)
        {
            if (handle == null)
                throw new ArgumentNullException("handle");

            OleDbCommand command = new OleDbCommand("DELETE FROM ProgramHandle WHERE id=@id", DefaultDbConnection);
            command.Parameters.Add(new OleDbParameter("id", OleDbType.VarChar) { Value = handle.ProgramId.ToString() });
            DefaultDbConnection.Open();
            try
            {
                command.ExecuteNonQuery();
            }
            finally
            {
                DefaultDbConnection.Close();
            }
        }

        internal static IEnumerable<ProgramHandle> GetProgramHandles()
        {
            OleDbCommand command = new OleDbCommand("SELECT id,InstanceFile,created FROM ProgramHandle", DefaultDbConnection);
            DefaultDbConnection.Open();
            using (IDataReader dataReader = command.ExecuteReader())
            {
                while (dataReader.Read())
                {
                    yield return Utility.Deserialize<ProgramHandle>(dataReader.GetString(1) + ".sf");
                }
            }
            DefaultDbConnection.Close();
        }

        internal static void UpdateProgramHandle(ProgramHandle programHandle)
        {
            if (!DbHelper.ExistsProgramHandle(programHandle))
            {
                DbHelper.InsertProgramHandle(programHandle);
            }
        }
    }
}
