﻿//-----------------------------------------------------------------------
// <copyright file="Program.cs">
// Copyright 温海雄. 2010 All Right Reserved
// </copyright>
// <author email="wen.haixiong@live.com" blog="http://www.cnblogs.com/cesium/">温海雄</author>
//-----------------------------------------------------------------------
namespace OpenSesame
{
    using System;
    using System.Collections.Generic;
    using System.Linq;

    class Program
    {
        static Guid _OpenSesameGuid = new Guid("32efd15d-a5de-44d5-97e8-e7a3bc5444c4");
        static IDictionary<Guid, ProgramStatement> _Programs = BuildPorgrams();

        private static IDictionary<Guid, ProgramStatement> BuildPorgrams()
        {
            Dictionary<Guid, ProgramStatement> dict = new Dictionary<Guid, ProgramStatement>();
            var openSesame = BuildOpenSesameProgram();
            dict.Add(_OpenSesameGuid, openSesame);
            return dict;
        }

        private static ProgramStatementBlock BuildOpenSesameProgram()
        {
            var openSesame = new ProgramStatementBlock();
            var printKey = new PrintKey();
            openSesame.Statements.Add(printKey);
            var read = new Read();
            openSesame.Statements.Add(read);
            var ifElse = new IfElse(new BooleanExpression((l, r) => ((PrintKey)l).Key == ((Read)r).Text, printKey, read));
            ifElse.Statements.Add(new PrintGreeting());
            openSesame.Statements.Add(ifElse);
            return openSesame;
        }

        static void Main(string[] args)
        {
            Console.WriteLine("progruam id: {0}", _OpenSesameGuid);
            MythicalRuntime runtime = new MythicalRuntime();
            string[] actionArray = GetActionArray();
            DoAction(runtime, actionArray);
            runtime.Shutdown();
            Console.WriteLine("press enter key to exit.");
            Console.ReadLine();
        }

        private static string[] GetActionArray()
        {
            Console.Write("choose run mode(all, part1, part2):");
            var mode = Console.ReadLine().ToLower();
            string[] actionArray1 = new string[] { 
                "",
                "OpenSesame.PrintKey"
            };
            string[] actionArray2 = new string[] 
            {
                "OpenSesame.Read.1",
                "OpenSesame.Read",
                "OpenSesame.IfElse",
                "OpenSesame.PrintGreeting"
            };
            switch (mode)
            {
                case "all":
                    return actionArray1.Concat(actionArray2).ToArray();
                case "part1":
                    return actionArray1;
                case "part2":
                    return actionArray2;
                default:
                    throw new NotSupportedException();
            }
        }

        private static void DoAction(MythicalRuntime runtime, string[] actionArray)
        {
            ProgramHandle handle;
            foreach (var action in actionArray)
            {
                switch (action)
                {
                    case "":
                        handle = runtime.RunProgram(_OpenSesameGuid, _Programs[_OpenSesameGuid]);
                        break;
                    case "OpenSesame.Read.1":
                        handle = runtime.GetProgramHandle(_OpenSesameGuid);
                        Console.Write("input read parameter: ");
                        var s = Console.ReadLine(); //模拟获得参数
                        handle.Resume("OpenSesame.Read.1", s);
                        break;
                    default:
                        handle = runtime.GetProgramHandle(_OpenSesameGuid);
                        handle.Resume(action, null);
                        break;
                }
            }
        }
    }
}
